@@ -18,6 +18,7 @@ import operator
 import locale
 import decimal
 import gettext
+from collections import defaultdict
 from decimal import Decimal
 
 from gi.repository import GLib, GObject, Gtk
@@ -50,6 +51,7 @@ class DictEntry(object):
         self.name = name
         self.definition = parent_widget.field.keys[name]
         self.parent_widget = parent_widget
+        self._signal_handlers = defaultdict(list)
         self.widget = self.create_widget()
         if self.definition.get('help'):
             parent_widget.tooltips.set_tip(
@@ -57,11 +59,16 @@ class DictEntry(object):
 
     def create_widget(self):
         widget = Gtk.Entry()
-        widget.connect('key-press-event', self.parent_widget.send_modified)
-        widget.connect('focus-out-event',
-            lambda w, e: self.parent_widget._focus_out())
+        self._signal_handlers[widget].append(
+            widget.connect('key-press-event',
+                self.parent_widget.send_modified))
+        self._signal_handlers[widget].append(
+            widget.connect('focus-out-event',
+                lambda w, e: self.parent_widget._focus_out()))
         widget.props.activates_default = True
-        widget.connect('activate', self.parent_widget.sig_activate)
+        self._signal_handlers[widget].append(
+            widget.connect('activate',
+                self.parent_widget.sig_activate))
         return widget
 
     def modified(self, value):
@@ -77,14 +84,21 @@ class DictEntry(object):
     def set_readonly(self, readonly):
         self.widget.set_editable(not readonly)
 
+    def disconnect_signals(self):
+        for widget, signal_ids in self._signal_handlers.items():
+            for handler_id in signal_ids:
+                widget.disconnect(handler_id)
+
 
 class DictBooleanEntry(DictEntry):
 
     def create_widget(self):
         widget = Gtk.CheckButton()
-        widget.connect('toggled', self.parent_widget.sig_activate)
-        widget.connect('focus-out-event', lambda w, e:
-            self.parent_widget._focus_out())
+        self._signal_handlers[widget].append(
+            widget.connect('toggled', self.parent_widget.sig_activate))
+        self._signal_handlers[widget].append(
+            widget.connect('focus-out-event',
+                lambda w, e: self.parent_widget._focus_out()))
         return widget
 
     def get_value(self):
@@ -102,6 +116,7 @@ class DictBooleanEntry(DictEntry):
         self.widget.set_sensitive(not readonly)
 
 
+
 class DictSelectionEntry(DictEntry):
     expand = False
     fill = False
@@ -112,30 +127,29 @@ class DictSelectionEntry(DictEntry):
         # customizing entry
         child = widget.get_child()
         child.props.activates_default = True
-        child.connect('changed', self.parent_widget.send_modified)
-        child.connect('focus-out-event',
-            lambda w, e: self.parent_widget._focus_out())
-        child.connect('activate',
-            lambda w: self.parent_widget._focus_out())
-        widget.connect('notify::active',
-            lambda w, e: self.parent_widget._focus_out())
+        self._signal_handlers[child].append(
+            child.connect('changed', self._changed))
+        self._signal_handlers[child].append(
+            child.connect('focus-out-event',
+                lambda w, e: self.parent_widget._focus_out()))
+        self._signal_handlers[child].append(
+            child.connect('activate',
+                lambda w: self.parent_widget._focus_out()))
         widget.connect(
             'scroll-event',
             lambda c, e: c.stop_emission_by_name('scroll-event'))
         selection_shortcuts(widget)
 
         # setting completion and selection
-        model = Gtk.ListStore(GObject.TYPE_STRING)
-        model.append(('',))
-        self._selection = {'': None}
+        model = Gtk.ListStore(GObject.TYPE_STRING, GObject.TYPE_PYOBJECT)
+        model.append(('', ""))
         width = 10
         selection = self.definition['selection']
         if self.definition.get('sort', True):
             selection.sort(key=operator.itemgetter(1))
         for value, name in selection:
             name = str(name)
-            self._selection[name] = value
-            model.append((name,))
+            model.append((name, value))
             width = max(width, len(name))
         widget.set_model(model)
         widget.set_entry_text_column(0)
@@ -145,29 +159,44 @@ class DictSelectionEntry(DictEntry):
         completion.set_model(model)
         child.set_completion(completion)
         completion.set_text_column(0)
+        completion.connect('match-selected', self.match_selected)
         return widget
 
+    def match_selected(self, completion, model, iter_):
+        value, = model.get(iter_, 1)
+        model = self.widget.get_model()
+        for i, selection in enumerate(model):
+            if selection[1] == value:
+                GLib.idle_add(self.widget.set_active, i)
+                break
+
     def get_value(self):
-        child = self.widget.get_child()
-        if not child:  # widget is destroyed
-            return
-        text = child.get_text()
-        value = None
-        if text:
-            for txt, val in list(self._selection.items()):
-                if not val:
-                    continue
-                if txt[:len(text)].lower() == text.lower():
-                    value = val
-                    if len(txt) == len(text):
-                        break
-        return value
+        active = self.widget.get_active()
+        if active < 0:
+            return None
+        else:
+            model = self.widget.get_model()
+            return model[active][1]
 
     def set_value(self, value):
-        values = dict(self.definition['selection'])
+        active = -1
+        model = self.widget.get_model()
+        for i, selection in enumerate(model):
+            if selection[1] == value:
+                active = i
+                break
         child = self.widget.get_child()
-        child.set_text(values.get(value, ''))
-        reset_position(child)
+        child.handler_block_by_func(self._changed)
+        try:
+            self.widget.set_active(active)
+            if active == -1:
+                # When setting no item GTK doesn't clear the entry
+                child.set_text('')
+        finally:
+            child.handler_unblock_by_func(self._changed)
+
+    def _changed(self, selection):
+        GLib.idle_add(self.parent_widget._focus_out)
 
     def set_readonly(self, readonly):
         self.widget.set_sensitive(not readonly)
@@ -186,12 +215,14 @@ class DictMultiSelectionEntry(DictEntry):
         self.tree = TreeViewControl()
         self.tree.set_model(model)
         self.tree.set_search_column(1)
-        self.tree.connect(
-            'focus-out-event', lambda w, e: self.parent_widget._focus_out())
+        self._signal_handlers[self.tree].append(
+            self.tree.connect('focus-out-event',
+                lambda w, e: self.parent_widget._focus_out()))
         self.tree.set_headers_visible(False)
         selection = self.tree.get_selection()
         selection.set_mode(Gtk.SelectionMode.MULTIPLE)
-        selection.connect('changed', self._changed)
+        self._signal_handlers[selection].append(
+            selection.connect('changed', self._changed))
         widget.add(self.tree)
 
         self.selection = self.definition['selection']
@@ -239,11 +270,15 @@ class DictIntegerEntry(DictEntry):
 
     def create_widget(self):
         widget = NumberEntry()
-        widget.connect('key-press-event', self.parent_widget.send_modified)
-        widget.connect('focus-out-event',
-            lambda w, e: self.parent_widget._focus_out())
+        self._signal_handlers[widget].append(
+            widget.connect('key-press-event',
+                self.parent_widget.send_modified))
+        self._signal_handlers[widget].append(
+            widget.connect('focus-out-event',
+                lambda w, e: self.parent_widget._focus_out()))
         widget.props.activates_default = True
-        widget.connect('activate', self.parent_widget.sig_activate)
+        self._signal_handlers[widget].append(
+            widget.connect('activate', self.parent_widget.sig_activate))
         return widget
 
     def get_value(self):
@@ -323,9 +358,12 @@ class DictDateTimeEntry(DictEntry):
             time_format = field.time_format(record)
             widget.props.date_format = date_format
             widget.props.time_format = time_format
-        widget.connect('key_press_event', self.parent_widget.send_modified)
-        widget.connect('focus-out-event', lambda w, e:
-            self.parent_widget._focus_out())
+        self._signal_handlers[widget].append(
+            widget.connect('key_press_event',
+                self.parent_widget.send_modified))
+        self._signal_handlers[widget].append(
+            widget.connect('focus-out-event',
+                lambda w, e: self.parent_widget._focus_out()))
         return widget
 
     def get_value(self):
@@ -355,9 +393,12 @@ class DictDateEntry(DictEntry):
         if record and field:
             format_ = field.date_format(record)
             widget.props.format = format_
-        widget.connect('key_press_event', self.parent_widget.send_modified)
-        widget.connect('focus-out-event', lambda w, e:
-            self.parent_widget._focus_out())
+        self._signal_handlers[widget].append(
+            widget.connect('key_press_event',
+                self.parent_widget.send_modified))
+        self._signal_handlers[widget].append(
+            widget.connect('focus-out-event',
+                lambda w, e: self.parent_widget._focus_out()))
         return widget
 
     def get_value(self):
@@ -483,6 +524,7 @@ class DictWidget(Widget):
                     focus = True
 
     def _sig_remove(self, button, key, modified=True):
+        self.fields[key].disconnect_signals()
         del self.fields[key]
         del self.buttons[key]
         for widget in self.rows[key]:
