@@ -27,6 +27,8 @@ from gnuhealth.gui.window.win_form import WinForm
 from gnuhealth.gui.window.view_form.screen import Screen
 from gnuhealth.common import file_selection, file_open, file_write
 import gnuhealth.common as common
+from gnuhealth.common import get_gdk_backend
+
 from gnuhealth.common.cellrendererbutton import CellRendererButton
 from gnuhealth.common.cellrenderertext import CellRendererText, \
     CellRendererTextCompletion
@@ -53,8 +55,8 @@ COLORS = {n: v for n, v in zip(
 
 def send_keys(renderer, editable, position, treeview):
     editable.connect('key_press_event', treeview.on_keypressed, renderer)
-    editable.editing_done_id = editable.connect('editing_done',
-            treeview.on_editing_done, renderer)
+    editable.editing_done_id = editable.connect(
+        'editing_done', treeview.on_editing_done, renderer)
     if isinstance(editable, Gtk.ComboBox):
         def changed(combobox):
             # "changed" signal is also triggered by text editing
@@ -322,14 +324,16 @@ class GenericText(Cell):
             cell.set_property('text', '')
 
         if self.view.editable:
-            readonly = self.attrs.get('readonly',
+            readonly = self.attrs.get(
+                'readonly',
                 field.get_state_attrs(record).get('readonly', False))
             if invisible:
                 readonly = True
 
             if isinstance(cell, CellRendererToggle):
                 cell.set_property('activatable', not readonly)
-            elif isinstance(cell,
+            elif isinstance(
+                cell,
                     (Gtk.CellRendererProgress, CellRendererButton,
                         Gtk.CellRendererPixbuf)):
                 pass
@@ -340,7 +344,8 @@ class GenericText(Cell):
                 cell.set_property('activatable', False)
         self._set_visual(cell, record)
 
-    def open_remote(self, record, create, changed=False, text=None,
+    def open_remote(
+        self, record, create, changed=False, text=None,
             callback=None):
         raise NotImplementedError
 
@@ -425,8 +430,7 @@ class Int(GenericText):
 class Boolean(GenericText):
     align = 0.5
 
-    def __init__(self, view, attrs=None,
-            renderer=None):
+    def __init__(self, view, attrs=None, renderer=None):
         if renderer is None:
             renderer = CellRendererToggle
         super(Boolean, self).__init__(view, attrs, renderer=renderer)
@@ -438,7 +442,8 @@ class Boolean(GenericText):
                 and not self.view.record.validate(self.view.get_fields())):
             renderer.stop_emission_by_name('toggled')
             return True
-        if not self.attrs.get('readonly',
+        if not self.attrs.get(
+            'readonly',
                 field.get_state_attrs(record).get('readonly', False)):
             value = record[self.attrs['name']].get_client(record)
             record[self.attrs['name']].set_client(record, int(not value))
@@ -633,7 +638,8 @@ class _BinarySave(_BinaryIcon):
         if self.attrs.get('filename'):
             filename_field = record.group.fields.get(self.attrs['filename'])
             filename = filename_field.get(record)
-        filename = file_selection(_('Save As...'), filename=filename,
+        filename = file_selection(
+            _('Save As...'), filename=filename,
             action=Gtk.FileChooserAction.SAVE)
         if filename:
             with open(filename, 'wb') as fp:
@@ -695,7 +701,8 @@ class _BinarySelect(_BinaryIcon):
         cell.set_property('pixbuf', pixbuf)
         field.state_set(record, states=['invisible', 'readonly'])
         invisible = field.get_state_attrs(record).get('invisible', False)
-        readonly = self.attrs.get('readonly',
+        readonly = self.attrs.get(
+            'readonly',
             field.get_state_attrs(record).get('readonly', False))
         if readonly and size:
             cell.set_property('visible', False)
@@ -829,6 +836,7 @@ class M2O(GenericText):
                 editable.set_icon_tooltip_text(pos, tooltip)
 
         def icon_press(editable, icon_pos, event):
+            editable.grab_focus()
             value = field.get(record)
             if not model:
                 return
@@ -849,7 +857,8 @@ class M2O(GenericText):
         changed(editable)
         return False
 
-    def open_remote(self, record, create=True, changed=False, text=None,
+    def open_remote(
+        self, record, create=True, changed=False, text=None,
             callback=None):
         field = record.group.fields[self.attrs['name']]
         model = self.get_model(record, field)
@@ -871,7 +880,8 @@ class M2O(GenericText):
         breadcrumb = list(self.view.screen.breadcrumb)
         breadcrumb.append(
             field.attrs.get('string') or common.MODELNAME.get(model))
-        screen = Screen(model, domain=domain, context=context,
+        screen = Screen(
+            model, domain=domain, context=context,
             mode=['form'], view_ids=self.attrs.get('view_ids', '').split(','),
             exclude_field=field.attrs.get('relation_field'),
             breadcrumb=breadcrumb)
@@ -889,7 +899,8 @@ class M2O(GenericText):
             screen.load([target_id])
             WinForm(screen, open_callback, save_current=True)
         else:
-            WinForm(screen, open_callback, new=True, save_current=True,
+            WinForm(
+                screen, open_callback, new=True, save_current=True,
                 rec_name=text)
 
     def search_remote(self, record, field, text, callback=None):
@@ -907,7 +918,8 @@ class M2O(GenericText):
                 field.set_client(record, value)
             if callback:
                 callback()
-        win = WinSearch(model, search_callback, sel_multi=False,
+        win = WinSearch(
+            model, search_callback, sel_multi=False,
             context=context, domain=domain,
             order=order, view_ids=self.attrs.get('view_ids', '').split(','),
             new=create_access, title=self.attrs.get('string'))
@@ -925,9 +937,11 @@ class M2O(GenericText):
         completion = get_completion(
             search=access['read'],
             create=self.attrs.get('create', True) and access['create'])
-        completion.connect('match-selected', self._completion_match_selected,
+        completion.connect(
+            'match-selected', self._completion_match_selected,
             record, field, model)
-        completion.connect('action-activated',
+        completion.connect(
+            'action-activated',
             self._completion_action_activated, record, field)
         entry.set_completion(completion)
         entry.connect('key-press-event', self._key_press, record, field)
@@ -968,10 +982,12 @@ class M2O(GenericText):
             entry.handler_unblock(entry.editing_done_id)
             entry.set_text(field.get_client(record))
         if index == 0:
-            self.open_remote(record, create=False, changed=True,
+            self.open_remote(
+                record, create=False, changed=True,
                 text=entry.get_text(), callback=callback)
         elif index == 1:
-            self.open_remote(record, create=True,
+            self.open_remote(
+                record, create=True,
                 text=entry.get_text(), callback=callback)
         else:
             entry.handler_unblock(entry.editing_done_id)
@@ -985,14 +1001,16 @@ class O2M(GenericText):
     align = 0.5
 
     def get_textual_value(self, record):
-        return '( ' + str(len(record[self.attrs['name']]
+        return '( ' + str(
+            len(record[self.attrs['name']]
                 .get_eval(record))) + ' )'
 
     def value_from_text(self, record, text, callback=None):
         if callback:
             callback()
 
-    def open_remote(self, record, create=True, changed=False, text=None,
+    def open_remote(
+        self, record, create=True, changed=False, text=None,
             callback=None):
         group = record.value[self.attrs['name']]
         field = record.group.fields[self.attrs['name']]
@@ -1006,7 +1024,8 @@ class O2M(GenericText):
         breadcrumb = list(self.view.screen.breadcrumb)
         breadcrumb.append(
             field.attrs.get('string') or common.MODELNAME.get(relation))
-        screen = Screen(relation, mode=['tree', 'form'],
+        screen = Screen(
+            relation, mode=['tree', 'form'],
             view_ids=self.attrs.get('view_ids', '').split(','),
             exclude_field=field.attrs.get('relation_field'),
             breadcrumb=breadcrumb)
@@ -1021,7 +1040,8 @@ class O2M(GenericText):
 
 class M2M(O2M):
 
-    def open_remote(self, record, create=True, changed=False, text=None,
+    def open_remote(
+        self, record, create=True, changed=False, text=None,
             callback=None):
         group = record.value[self.attrs['name']]
         field = record.group.fields[self.attrs['name']]
@@ -1032,7 +1052,8 @@ class M2M(O2M):
         breadcrumb = list(self.view.screen.breadcrumb)
         breadcrumb.append(
             field.attrs.get('string') or common.MODELNAME.get(relation))
-        screen = Screen(relation, mode=['tree', 'form'],
+        screen = Screen(
+            relation, mode=['tree', 'form'],
             view_ids=self.attrs.get('view_ids', '').split(','),
             exclude_field=field.attrs.get('relation_field'),
             breadcrumb=breadcrumb)
@@ -1041,7 +1062,8 @@ class M2M(O2M):
         def open_callback(result):
             if callback:
                 callback()
-        WinForm(screen, open_callback, view_type='tree', domain=domain,
+        WinForm(
+            screen, open_callback, view_type='tree', domain=domain,
             context=context)
 
 
@@ -1084,10 +1106,12 @@ class Selection(GenericText, SelectionMixin, PopdownMixin):
     def editing_started(self, cell, editable, path):
         super(Selection, self).editing_started(cell, editable, path)
         record, field = self._get_record_field_from_path(path)
-        # Combobox does not emit remove-widget when focus is changed
-        self.editable.connect(
-            'editing-done',
-            lambda *a: self.editable.emit('remove-widget'))
+        gdk_backend = get_gdk_backend()
+        if gdk_backend == 'x11':
+            # Combobox does not emit remove-widget when focus is changed
+            self.editable.connect(
+                'editing-done',
+                lambda *a: self.editable.emit('remove-widget'))
 
         selection_shortcuts(editable)
 
@@ -1095,6 +1119,15 @@ class Selection(GenericText, SelectionMixin, PopdownMixin):
             return self.set_value(editable, record, field)
         editable.get_child().connect('activate', set_value)
         editable.get_child().connect('focus-out-event', set_value)
+
+        if gdk_backend != 'x11':
+            # For non X11 backend we can rely the focus-out-event to be sent
+            # correctly
+            def remove_entry(entry, event):
+                editable.emit('editing-done')
+                editable.emit('remove-widget')
+            editable.get_child().connect('focus-out-event', remove_entry)
+
         editable.connect('changed', set_value)
 
         self.update_selection(record, field)
@@ -1266,7 +1299,8 @@ class ProgressBar(Cell):
         cell.set_property('value', value * 100)
         self._set_visual(cell, record)
 
-    def open_remote(self, record, create, changed=False, text=None,
+    def open_remote(
+        self, record, create, changed=False, text=None,
             callback=None):
         raise NotImplementedError
 
@@ -1298,7 +1332,7 @@ class Button(Cell):
         states = record.expr_eval(self.attrs.get('states', {}))
         invisible = states.get('invisible', False)
         cell.set_property('visible', not invisible)
-        readonly = states.get('readonly', False)
+        readonly = record.readonly or states.get('readonly', False)
         cell.set_property('sensitive', not readonly)
         parent = record.parent if record else None
         while parent:
